#include <SoftwareSerial.h>
#include <LiquidCrystal.h>
#include <DHT.h>
#include <DHT_U.h>

#define BT_SERIAL_TX 10
#define BT_SERIAL_RX 13
#define DHTPIN A0
#define DHTTYPE DHT11
#define MOTOR 6
#define Buzzer 9
#define FLAME_A A3
#define FLAME_D 8
#define JOYSTICK 7

// Initialise the software serial port
SoftwareSerial BluetoothSerial(BT_SERIAL_TX, BT_SERIAL_RX);

// define variables
int dig[4];
int temp_intreg[2];
int temp_zecimal[2];
int set_hour_first[1];
int set_hour_second[1];
int set_min_first[1];
int set_min_second[1];
float ref_temp = 0.0;
int input_hour = 0;
int input_min = 0;
int seconds_contor = 0;
int position = 0;


// initialize lcd pins
LiquidCrystal lcd(12, 11, 5, 4, 3, 2);

DHT dht(DHTPIN, DHTTYPE);

// setup function
void setup() {

  // initialize pins
  Serial.begin(9600);
  pinMode(MOTOR, OUTPUT);
  pinMode(Buzzer, OUTPUT);
  pinMode(FLAME_D, INPUT);
  BluetoothSerial.begin(9600);

  // display message
  lcd.begin(16, 2);
  lcd.setCursor(3, 0);
  lcd.print("PM Project");
  lcd.setCursor(1, 1);
  lcd.print("By Andreea C.");
  delay(3000);
  lcd.clear();


  // display message for password
  lcd.begin(16, 2);
  lcd.print("Enter password:");
  lcd.setCursor(6, 1);
  lcd.print("0000");
  lcd.cursor();
  lcd.noBlink();
  lcd.setCursor(6, 1);

  // initialize vectors
  for (int i = 0; i <= 3; i++) {
    dig[i] = 0;
  }

  for (int i = 0; i <= 1; i++) {
    temp_intreg[i] = 0;
    temp_zecimal[i] = 0;
  }

  set_hour_first[1] = 0;
  set_hour_second[1] = 0;
  set_min_first[1] = 0;
  set_min_first[1] = 0;
  
  // blink LED on board
  digitalWrite(JOYSTICK, LOW);
  delay(500);
  digitalWrite(JOYSTICK, HIGH);

  position = 0;

  // enter password
  if (digitalRead(JOYSTICK) == HIGH) {
    while (digitalRead(JOYSTICK) == HIGH) {
      int moveX = analogRead(A1);
      int moveY = analogRead(A2);
      float angleX = moveX * (180.0 / 1023.0);
      float angleY = moveY * (180.0 / 1023.0);
      if (angleX > 100) {
        moveRight();
        delay(500);
      } else if (angleX < 70) {
        moveLeft();
        delay(500);
      } else {
        if (angleY > 105) {
          decrease(position);
          delay(250);
        } else if (angleY < 75) {
          increase(position);
          delay(250);
        }
      }
    }
  }

  lcd.clear();

  // check passowrd;
  // if it's correct, the reference temperature is set
  if (checkPassword() == 1) {
    delay(2000);
    Serial.println("OK");
    lcd.clear();
    lcd.setCursor(0, 0);
    lcd.print("Select ref temp");
    lcd.setCursor(0, 1);
    lcd.print("in *C");
    lcd.cursor();
    lcd.setCursor(6, 1);
    lcd.print("00.00");
    lcd.setCursor(6, 1);
    position = 0;


    if (digitalRead(JOYSTICK) == HIGH) {
      while (digitalRead(JOYSTICK) == HIGH) {
        int moveX = analogRead(A1);
        int moveY = analogRead(A2);
        float angleX =
          moveX * (180.0 / 1023.0);
        float angleY = moveY * (180.0 / 1023.0);
        if (angleX > 100) {
          moveRight_temp_intreg();
          delay(500);
        } else if (angleX < 70) {
          moveLeft_temp_intreg();
          delay(500);
        } else {
          if (angleY > 105) {
            decrease_temp_intreg(position);
            delay(250);
          } else if (angleY < 75) {
            increase_temp_intreg(position);
            delay(250);
          }
        }
      }
    }


    // the whole part introduced for temperature
    // if it's correct, set the decimal part for the temperature
    if (check_temp_intreg() == 1) {
      Serial.println("whole part OK");
      delay(1000);
      lcd.cursor();
      lcd.setCursor(9, 1);
      position = 0;

      if (digitalRead(JOYSTICK) == HIGH) {
        while (digitalRead(JOYSTICK) == HIGH) {
          int moveX = analogRead(A1);
          int moveY = analogRead(A2);
          float angleX = moveX * (180.0 / 1023.0);
          float angleY = moveY * (180.0 / 1023.0);
          if (angleX > 100) {
            moveRight_temp_zecimal();
            delay(500);
          } else if (angleX < 70) {
            moveLeft_temp_zecimal();
            delay(500);
          } else {
            if (angleY > 105) {
              decrease_temp_zecimal(position);
              delay(250);
            } else if (angleY < 75) {
              increase_temp_zecimal(position);
              delay(250);
            }
          }
        }
      }
    }

    // check the decimal part for the temperature
    // and set time(HH:MM)
    if (check_temp_zecimal() == 1) {
      delay(1000);
      Serial.print("decimal part OK");
      lcd.clear();
      lcd.setCursor(0, 0);
      lcd.noCursor();
      lcd.print("Ref temp. has");
      lcd.setCursor(0, 1);
      lcd.print("been set.");
      //lcd.cursor();
      delay(2000);
      lcd.clear();

      // set time
      lcd.setCursor(0, 0);
      lcd.print("Set time:");
      lcd.cursor();
      lcd.setCursor(6, 1);
      lcd.print("HH:MM");
      lcd.setCursor(6, 1);
      position = 0;

      if (digitalRead(JOYSTICK) == HIGH) {
        while (digitalRead(JOYSTICK) == HIGH) {
          int moveX = analogRead(A1);
          int moveY = analogRead(A2);
          float angleX = moveX * (180.0 / 1023.0);
          float angleY = moveY * (180.0 / 1023.0);
          if (angleX > 100) { ;
            delay(500);
          } else if (angleX < 70) { ;
            delay(500);
          } else {
            if (angleY > 105) {
              decrease_hour_first(position);
              delay(250);
            } else if (angleY < 75) {
              increase_hour_first(position);
              delay(250);
            }
          }
        }
      }
    }


    // check first digit for hour
    if (check_hour_first() == 1) {
      delay(1000);
      lcd.cursor();
      lcd.setCursor(JOYSTICK, 1);
      position = 0;


      if (digitalRead(JOYSTICK) == HIGH) {
        while (digitalRead(JOYSTICK) == HIGH) {
          int moveX = analogRead(A1);
          int moveY = analogRead(A2);
          float angleX = moveX * (180.0 / 1023.0);
          float angleY = moveY * (180.0 / 1023.0);
          if (angleX > 100) { ;
            delay(500);
          } else if (angleX < 70) { ;
            delay(500);
          } else {
            if (angleY > 105) {
              decrease_hour_second(position);
              delay(250);
            } else if (angleY < 75) {
              increase_hour_second(position);
              delay(250);
            }
          }
        }
      }
    }

    // check second digit for hour
    if (check_hour_second() == 1) {
      delay(1000);
      lcd.cursor();
      lcd.setCursor(9, 1);
      position = 0;


      if (digitalRead(JOYSTICK) == HIGH) {
        while (digitalRead(JOYSTICK) == HIGH) {
          int moveX = analogRead(A1);
          int moveY = analogRead(A2);
          float angleX =
            moveX * (180.0 / 1023.0);
          float angleY = moveY * (180.0 / 1023.0);
          if (angleX > 100) { ;
            delay(500);
          } else if (angleX < 70) { ;
            delay(500);
          } else {
            if (angleY > 105) {
              decrease_minute_first(position);
              delay(250);
            } else if (angleY < 75) {
              increase_minute_first(position);
              delay(250);
            }
          }
        }
      }


      // check first digit for minutes
      if (check_min_first() == 1) {
        delay(1000);
        lcd.cursor();
        lcd.setCursor(10, 1);
        position = 0;


        if (digitalRead(JOYSTICK) == HIGH) {
          while (digitalRead(JOYSTICK) == HIGH) {
            int moveX = analogRead(A1);
            int moveY = analogRead(A2);
            float angleX =
              moveX * (180.0 / 1023.0);
            float angleY = moveY * (180.0 / 1023.0);
            if (angleX > 100) { ;
              delay(500);
            } else if (angleX < 70) { ;
              delay(500);
            } else {
              if (angleY > 105) {
                decrease_minute_second(position);
                delay(250);
              } else if (angleY < 75) {
                increase_minute_second(position);
                delay(250);
              }
            }
          }
        }

      }
    }

    // check second digit for minutes
    if (check_min_second() == 1) {


      delay(1000);
      Serial.print("min OK");
      lcd.clear();
      lcd.setCursor(0, 0);
      lcd.noCursor();
      lcd.print("Time has been");
      lcd.setCursor(0, 1);
      lcd.print("set.");
      //lcd.cursor();
      delay(2000);
      lcd.clear();
    }


  }

  // compute ref temp
  // compute hour and minutes
  ref_temp = temp_intreg[0] * 10 + temp_intreg[1] + 0.1 * temp_zecimal[0] +
         0.01 * temp_zecimal[1];
  input_hour = set_hour_first[0] * 10 + set_hour_second[0];
  input_min = set_min_first[0] * 10 + set_min_second[0];
  Serial.println(ref_temp);
  Serial.println(input_hour);
  Serial.println(input_min);


  lcd.clear();
  dht.begin();

  // display Temp(T) and Humidity(H)
  lcd.setCursor(0, 0);
  lcd.print("T:");
  lcd.setCursor(0, 1);
  lcd.print("H:");
  lcd.setCursor(8, 1);
  lcd.print("%");

  delay(1000);

}


// loop function
void loop() {

  
  lcd.noCursor();

  float h = dht.readHumidity();
  float t = dht.readTemperature();

  Serial.println(
    "Humidity: " + String(h) + "%, Temperature: " + String(t) + "°C");
  BluetoothSerial.println(
    "Humidity: " + String(h) + "%, Temperature: " + String(t) + "°C");


  // check H&T sensor
  if (isnan(t) && (h)) {
    lcd.clear();
    lcd.setCursor(4, 1);
    lcd.print("|SENSOR|");
    lcd.setCursor(4, 0);
    lcd.print("|ERROR |");
    delay(1000);
    return;
  }

  // values for dc motor
  if (t < ref_temp) {
    analogWrite(MOTOR, LOW);
    BluetoothSerial.println("The fan is off now.");
  }
  if ((t - ref_temp < 2.00) && (t - ref_temp >= 0)) {
    BluetoothSerial.println("The fan is on now, speed is on stage 1.");
    analogWrite(MOTOR, 100);
  } else if ((t - ref_temp > 2.00) && (t - ref_temp < 5.00)) {
    BluetoothSerial.println("The fan is on now, speed is on stage 2.");
    analogWrite(MOTOR, 150);
  } else if (t - ref_temp > 5.00) {
    BluetoothSerial.println("The fan is on now, speed is on stage 3.");
    analogWrite(MOTOR, 200);
  }

  // print values for temp and humidity
  lcd.setCursor(3, 0);
  lcd.print(t);
  lcd.setCursor(3, 1);
  lcd.print(h);

  // compute hour, min, sec
  if (seconds_contor < 60) {
    delay(1000);
    seconds_contor = seconds_contor + 1;
  } else {
    lcd.setCursor(14, 1);
    lcd.print(00);
    seconds_contor = 00;
  }

  if (seconds_contor > 59) {
    input_min = input_min + 1;
  } else {
    input_min = input_min;
  }

  if (input_min < 60) {
    input_min = input_min;
    input_hour = input_hour;
  } else {
    input_min = 0;
    input_hour = input_hour + 1;
  }

  if (input_hour < 24) {
    input_hour = input_hour;
  } else {
    input_hour = 0;
  }

  // display hour, min, sec
  lcd.setCursor(10, 0);
  lcd.print(input_hour);
  lcd.print(":");
  lcd.print(input_min);
  lcd.print(":");
  if (seconds_contor < 10) {

    lcd.setCursor(15, 1);
    lcd.print(seconds_contor);
  } else if (seconds_contor >= 10) {

    lcd.setCursor(14, 1);
    lcd.print(seconds_contor);
  }

  int analogVal = analogRead(FLAME_A);
  int digitalVal = digitalRead(FLAME_D);

  Serial.print("Fire analog val: ");
  Serial.print(analogVal);
  Serial.print("\t");
  Serial.print("Fire digital val: ");
  Serial.print(digitalVal);
  Serial.print("\t");

  if (analogVal < 35) {
    Serial.println("Fire");
    digitalWrite(Buzzer, HIGH);
    delay(100);
    digitalWrite(Buzzer, LOW);
    BluetoothSerial.println("Alert!!! Fire!!!");

  } else {
    digitalWrite(Buzzer, LOW);
    BluetoothSerial.println("It's ok, no fire detected.");
  }
  delay(1500);

}


// aux functions for password
void moveRight() {
  if (position == 3) {
    lcd.setCursor(6, 1);
    position = 0;
  } else {
    int a = position + 7;
    lcd.setCursor(a, 1);
    position = position + 1;
  }
}

void moveLeft() {
  if (position == 0) {
    lcd.setCursor(9, 1);
    position = 3;
  } else {
    int b = position + 5;
    lcd.setCursor(b, 1);
    position = position - 1;
  }
}

void increase(int position) {
  if (dig[position] == 9) {
    dig[position] = 0;
  } else {
    dig[position] = dig[position] + 1;
  }
  int c = position + 6;
  lcd.setCursor(c, 1);
  lcd.print(dig[position]);
  lcd.setCursor(c, 1);
}

void decrease(int position) {
  if (dig[position] == 0) {
    dig[position] = 9;
  } else {
    dig[position] = dig[position] - 1;
  }
  int c = position + 6;
  lcd.setCursor(c, 1);
  lcd.print(dig[position]);
  lcd.setCursor(c, 1);
}


// aux functions for ref temp
void moveRight_temp_intreg() {
  if (position == 1) {
    lcd.setCursor(6, 1);
    position = 0;
  } else {
    int a = position + 7;
    lcd.setCursor(a, 1);
    position = position + 1;
  }
}

void moveLeft_temp_intreg() {
  if (position == 0) {
    lcd.setCursor(9, 1);
    position = 1;
  } else {
    int b = position + 5;
    lcd.setCursor(b, 1);
    position = position - 1;
  }
}

void increase_temp_intreg(int position) {
  if (temp_intreg[position] == 9) {
    temp_intreg[position] = 0;
  } else {
    temp_intreg[position] = temp_intreg[position] + 1;
  }
  int c = position + 6;
  lcd.setCursor(c, 1);
  lcd.print(temp_intreg[position]);
  lcd.setCursor(c, 1);
}

void decrease_temp_intreg(int position) {
  if (temp_intreg[position] == 0) {
    temp_intreg[position] = 9;
  } else {
    temp_intreg[position] = temp_intreg[position] - 1;
  }
  int c = position + 6;
  lcd.setCursor(c, 1);
  lcd.print(temp_intreg[position]);
  lcd.setCursor(c, 1);
}


void moveRight_temp_zecimal() {
  if (position == 1) {
    lcd.setCursor(9, 1);
    position = 0;
  } else {
    lcd.setCursor(10, 1);
    position = 1;
  }
}

void moveLeft_temp_zecimal() {
  if (position == 0) {
    lcd.setCursor(10, 1);
    position = 1;
  } else {
    lcd.setCursor(9, 1);
    position = 0;
  }
}


void increase_temp_zecimal(int position) {
  if (temp_zecimal[position] == 9) {
    temp_zecimal[position] = 0;
  } else {
    temp_zecimal[position] = temp_zecimal[position] + 1;
  }
  int c = position + 9;
  lcd.setCursor(c, 1);
  lcd.print(temp_zecimal[position]);
  lcd.setCursor(c, 1);
}

void decrease_temp_zecimal(int position) {
  if (temp_zecimal[position] == 0) {
    temp_zecimal[position] = 9;
  } else {
    temp_zecimal[position] = temp_zecimal[position] - 1;
  }
  int c = position + 9;
  lcd.setCursor(c, 1);
  lcd.print(temp_zecimal[position]);
  lcd.setCursor(c, 1);
}

// aux functions for the first digit of the hour
void increase_hour_first(int position) {
  if (set_hour_first[position] == 2) {
    set_hour_first[position] = 0;
  } else {
    set_hour_first[position] = set_hour_first[position] + 1;
  }
  int c = position + 6;
  lcd.setCursor(c, 1);
  lcd.print(set_hour_first[position]);
  lcd.setCursor(c, 1);
}

void decrease_hour_first(int position) {
  if (set_hour_first[position] == 0) {
    set_hour_first[position] = 2;
  } else {
    set_hour_first[position] = set_hour_first[position] - 1;
  }
  int c = position + 6;
  lcd.setCursor(c, 1);
  lcd.print(set_hour_first[position]);
  lcd.setCursor(c, 1);
}

// aux functions for the second digit of the hour
void increase_hour_second(int position) {
  if (set_hour_second[position] == 9) {
    set_hour_second[position] = 0;
  } else {
    set_hour_second[position] = set_hour_second[position] + 1;
  }
  int c = position + 7;
  lcd.setCursor(c, 1);
  lcd.print(set_hour_second[position]);
  lcd.setCursor(c, 1);
}

void decrease_hour_second(int position) {
  if (set_hour_second[position] == 0) {
    set_hour_second[position] = 9;
  } else {
    set_hour_second[position] = set_hour_second[position] - 1;
  }
  int c = position + 7;
  lcd.setCursor(c, 1);
  lcd.print(set_hour_second[position]);
  lcd.setCursor(c, 1);
}

// aux functions for the first digit of the minutes
void increase_minute_first(int position) {
  if (set_min_first[position] == 5) {
    set_min_first[position] = 0;
  } else {
    set_min_first[position] = set_min_first[position] + 1;
  }
  int c = position + 9;
  lcd.setCursor(c, 1);
  lcd.print(set_min_first[position]);
  lcd.setCursor(c, 1);
}

void decrease_minute_first(int position) {
  if (set_min_first[position] == 0) {
    set_min_first[position] = 5;
  } else {
    set_min_first[position] = set_min_first[position] - 1;
  }
  int c = position + 9;
  lcd.setCursor(c, 1);
  lcd.print(set_min_first[position]);
  lcd.setCursor(c, 1);
}

// aux functions for the second digit of the minutes
void increase_minute_second(int position) {
  if (set_min_second[position] == 9) {
    set_min_second[position] = 0;
  } else {
    set_min_second[position] = set_min_second[position] + 1;
  }
  int c = position + 10;
  lcd.setCursor(c, 1);
  lcd.print(set_min_second[position]);
  lcd.setCursor(c, 1);
}

void decrease_minute_second(int position) {
  if (set_min_second[position] == 0) {
    set_min_second[position] = 9;
  } else {
    set_min_second[position] = set_min_second[position] - 1;
  }
  int c = position + 10;
  lcd.setCursor(c, 1);
  lcd.print(set_min_second[position]);
  lcd.setCursor(c, 1);
}


// check function for password
int checkPassword() {
  int toreturn = 0;
  if (dig[0] == 1 && dig[1] == 2 && dig[2] == 3 && dig[3] == 4) {
    lcd.noCursor();
    lcd.setCursor(4, 1);
    lcd.print("Correct!");
    toreturn = 1;
  } else {
    lcd.noCursor();
    lcd.clear();
    lcd.setCursor(5, 1);
    lcd.print("Wrong!");
    delay(1500);
    setup();
    toreturn = 0;
  }
  return toreturn;
}



// check function for the whole part of the temperature
int check_temp_intreg() {
  int toreturn = 0;
  if (temp_intreg[0] >= 0 && temp_intreg[1] >= 0 && temp_intreg[0] <= 4 &&
    temp_intreg[1] <= 9) {
    toreturn = 1;
  } else {
    lcd.noCursor();
    lcd.clear();
    lcd.print("Wrong temp!");
    delay(1500);
    setup();
  }
  return toreturn;
}


// check function for the decimal part of the temperature
int check_temp_zecimal() {
  int toreturn = 0;
  if (temp_zecimal[0] >= 0 && temp_zecimal[1] >= 0) {
    toreturn = 1;
  } else {
    lcd.noCursor();
    lcd.clear();
    lcd.print("Wrong temp!");
    delay(1500);
    setup();
  }
  return toreturn;
}


// check function for the first digit of the hour
int check_hour_first() {
  int toreturn = 0;
  if (set_hour_first[0] >= 0 && set_hour_first[0] <= 2) {
    toreturn = 1;
  } else {
    lcd.noCursor();
    lcd.clear();
    lcd.print("Wrong Hour!");
    delay(1500);
    setup();

  }
  return toreturn;
}


// check function for the second digit of the hour
int check_hour_second() {
  int toreturn = 0;
  if (set_hour_second[0] >= 0 && set_hour_second[0] <= 9 &&
    set_hour_first[0] >= 0 && set_hour_first[0] <= 1) {
    toreturn = 1;
  } else if (set_hour_second[0] >= 0 && set_hour_second[0] <= 4 &&
         set_hour_first[0] == 2) {
    toreturn = 1;
  } else {
    lcd.noCursor();
    lcd.clear();
    lcd.print("Wrong Hour!");
    delay(1500);
    setup();

  }
  return toreturn;
}

// check function for the first digit of the minutes
int check_min_first() {
  int toreturn = 0;
  if (set_min_first[0] >= 0 && set_min_first[0] <= 5) {
    toreturn = 1;
  } else {
    lcd.noCursor();
    lcd.clear();
    lcd.print("Wrong Min!");
    delay(1500);
    setup();

  }
  return toreturn;
}

// check function for the second digit of the minutes
int check_min_second() {
  int toreturn = 0;
  if (set_min_second[0] >= 0 && set_min_second[0] <= 9) {
    toreturn = 1;
  } else {
    lcd.noCursor();
    lcd.clear();
    lcd.print("Wrong Min!");
    delay(1000);
    setup();

  }
  return toreturn;
}
